#include <errno.h>

#include <stdio.h>
#include <stdlib.h>

#include <unistd.h>

#include <libgsloop.h>


static int client_event(gsloop_client_t	*client,
			gsloop_event_t	*event,
			void		*data)
{
	ssize_t		bytes_read;
	char		buf[255];
	int		err = 0;

	if (event->revents & POLLIN) {
		bytes_read = read(event->fd, buf, sizeof(buf));
		if (bytes_read == -1 || bytes_read == 0) {
			gsloop_client_close(client);
			return -errno;
		}
		buf[bytes_read] = 0x0;
		fprintf(stdout, "%s", buf);
	} else {
		switch (event->revents) {
		case POLLOUT:
			printf("POLLOUT\n");
			break;
		default:
			printf("Event: %d\n", event->revents);
			break;
		}
	}

	return err;
}


static int client_attach(gsloop_client_t	*client,
			 void			*data)
{
	static int	i;

	printf("client attached %d\n", i++);
	
	return 0;
}


int main()
{
	gsloop_t		gsloop;
	gsloop_attr_t		gsloop_attr = {
		.max_fd	= 32,
	};
	gsloop_server_attr_t	gsloop_server_attr = {
		.max_clients	= 2,
		.client_event	= client_event,
		.client_attach	= client_attach,
	};
	int			err, loop = 20;

	err = gsloop_init(&gsloop, &gsloop_attr);

	err = gsloop_tcp_server_create(&gsloop, 4444, &gsloop_server_attr);

	while (err == 0 && loop--) {
		err = gsloop_dispatch(&gsloop, -1);
		printf("%d\n", loop);
	}

	gsloop_destroy(&gsloop);

	exit (EXIT_SUCCESS);
}
